/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.l2j.gameserver.skills.effects;

import net.sf.l2j.gameserver.ai.CtrlEvent;
import net.sf.l2j.gameserver.model.L2Character;
import net.sf.l2j.gameserver.model.L2Effect;
import net.sf.l2j.gameserver.model.L2Summon;
import net.sf.l2j.gameserver.model.actor.instance.L2EffectPointInstance;
import net.sf.l2j.gameserver.model.actor.instance.L2PcInstance;
import net.sf.l2j.gameserver.model.actor.instance.L2PlayableInstance;
import net.sf.l2j.gameserver.network.SystemMessageId;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.skills.Env;

/**
 * @author Forsaiken
 */

final class EffectSignetAntiSummon extends L2Effect
{
	private L2EffectPointInstance _actor;
	
	public EffectSignetAntiSummon(Env env, EffectTemplate template)
    {
        super(env, template);
    }
    
    @Override
    public EffectType getEffectType()
    {
        return EffectType.SIGNET_GROUND;
    }
    
    @Override
 	public void onStart()
 	{
 		_actor = (L2EffectPointInstance)getEffected();
 	}
    
    @Override
    public boolean onActionTime()
    {
    	if (getCount() == getTotalCount() - 1) return true; // do nothing first time
    	int mpConsume = getSkill().getMpConsume();
        
        for (L2Character cha : _actor.getKnownList().getKnownCharactersInRadius(getSkill().getSkillRadius()))
        {
            if (cha == null)
                continue;
            
            if (cha instanceof L2PlayableInstance)
            {
                L2PcInstance owner = null;
                
                if (cha instanceof L2Summon)
                    owner = ((L2Summon)cha).getOwner();
                else
                    owner = (L2PcInstance)cha;
                
                if (owner != null && owner.getPet() != null)
                {
                    if (mpConsume > getEffector().getCurrentMp())
                    {
                    	getEffector().sendPacket(new SystemMessage(SystemMessageId.SKILL_REMOVED_DUE_LACK_MP));
                        return false;
                    }
                    else
                    	getEffector().reduceCurrentMp(mpConsume);
                    
                    owner.getPet().unSummon(owner);
                    owner.getAI().notifyEvent(CtrlEvent.EVT_ATTACKED, getEffector());
                }
            }
        }
        return true;
    }
    
    @Override
    public void onExit()
    {
    	if (_actor != null)
        {
            _actor.deleteMe();
        }
    }
}
